unit F_RubberBanding;

(*
Notes on this program...
- the UserInterface does not use any of Delphi's UI sophistication, eg actions,
  so that you can concentrate on the effects of the code rather than on what the UI is doing
- the fonts on the panels for the UI are set to fsBold, purely for legibility when projected at BUG
- not all properties/methods/values are shown here, but they usually follow the same principles
- anything in brackets is not part of TCanvas, but a specially-written utility routine
*)


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, Spin, ComCtrls, ColorGrd;


type
  TfrmRubberBanding = class(TForm)
    pnlHeader: TPanel;
    pnlOnScreen: TPanel;
    pntOnScreen: TPaintBox;
    pnlMethods: TPanel;
    rdoMethods: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pntOnScreenMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure pntOnScreenMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure pntOnScreenMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmRubberBanding: TfrmRubberBanding;

implementation

uses
  BugColourControl,
  BugCommon;

{$R *.DFM}


var
  MarqueeRect: TRect;
  LinePoints: array of TPoint;

const
  // some constants to give human names to the ItemIndex values in rdoMethods
  iiRectangular = 0;
  iiSingleLine  = 1;
  iiLotsOfLines = 2;

procedure TfrmRubberBanding.FormCreate(Sender: TObject);
begin
  // and since we're not gonna do any real drawing on pntOnScreen, only marquee-stuff, set up the pen/brush just for that
  pntOnScreen.Canvas.Brush.Style := bsClear;
  pntOnScreen.Canvas.Pen.Style := psDot;
  pntOnScreen.Canvas.Pen.Mode := pmXOR;
end;

procedure TfrmRubberBanding.FormDestroy(Sender: TObject);
begin
  LinePoints:= nil;  // the equivalent of free for a dynamic array
end;

procedure TfrmRubberBanding.pntOnScreenMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft
  then begin
          if rdoMethods.ItemIndex = iiRectangular
          then begin
                 // get rid of any existing rectangle on-screen
                 if not IsRectEmpty(MarqueeRect)   // Windows API, but simple to port to Kylix
                 then pntOnScreen.Canvas.DrawFocusRect(MarqueeRect);
                 // and start off a new rectangle whose top-left is the mouse XY
                 MarqueeRect.Left := X;
                 MarqueeRect.Top := Y;
                 MarqueeRect.BottomRight := MarqueeRect.TopLeft;
               end // RectangleMethods
          else if rdoMethods.ItemIndex = iiSingleLine
               then begin
                      // ensure there's a starting Point
                      if Length(LinePoints) = 0
                      then begin
                            SetLength(LinePoints, 1);
                            LinePoints[0] := Point(X, Y);
                           end;
                      // and put the current Point in
                      SetLength(LinePoints, Succ(Length(LinePoints)));
                      LinePoints[high(LinePoints)] := Point(X, Y);
                    end // LineMethods
          else if rdoMethods.ItemIndex = iiLotsOfLines
               then begin
                      // ensure there's a starting Point
                      if Length(LinePoints) = 0
                      then begin
                            SetLength(LinePoints, 1);
                            LinePoints[0] := Point(X, Y);
                           end;
                      // and put the current Point in
                      SetLength(LinePoints, Succ(Length(LinePoints)));
                      LinePoints[high(LinePoints)] := Point(X, Y);
                    end; // iiLotsOfLines
       end; //if Button = mbLeft
end;

procedure TfrmRubberBanding.pntOnScreenMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if (ssLeft in Shift)
      or
     ((rdoMethods.ItemIndex = iiLotsOfLines) and (Length(LinePoints) > 1))
  then begin
               if rdoMethods.ItemIndex in [iiSingleLine, iiLotsOfLines]
               then begin
                      if Length(LinePoints) > 1
                      then begin
                              // remember, the pntOnScreen.Canvas.Pen is already clBlack, 1, psDot, pmXOR
                              // wipe the previous line
                              pntOnScreen.Canvas.PolyLine(Copy(LinePoints, Pred(high(LinePoints)), 2));
                              // put the new line in
                              LinePoints[high(LinePoints)] := Point(X, Y);
                              pntOnScreen.Canvas.PolyLine(Copy(LinePoints, Pred(high(LinePoints)), 2));
                           end; // there's more than 1 point
                    end // LineMethods
               else if rdoMethods.ItemIndex = iiRectangular
                    then begin
                           if not IsRectEmpty(MarqueeRect)   // Windows API, but simple to port to Kylix
                           then pntOnScreen.Canvas.DrawFocusRect(MarqueeRect);
                           // should ensure that the bottomright IS right and down from topleft, else Windows thinks it's an invalid rect
                           MarqueeRect.Right := X;
                           MarqueeRect.Bottom := Y;
                           pntOnScreen.Canvas.DrawFocusRect(MarqueeRect);
                         end; // RectangleMethods
             end; // if ssLeft or lines
end;

procedure TfrmRubberBanding.pntOnScreenMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft
  then begin
         // get rid of the latest focusrect
         if not IsRectEmpty(MarqueeRect)   // Windows API, but simple to port to Kylix
         then pntOnScreen.Canvas.DrawFocusRect(MarqueeRect);
         // ... now decide exactly what to do in response to this mouseup...
         // we've probably done something so clear the marquee definition
         if rdoMethods.ItemIndex = iiRectangular
         then MarqueeRect:= Rect(0,0,0,0)
         else if rdoMethods.ItemIndex = iiSingleLine
              then SetLength(LinePoints, 0);
       end // mbLeft
  else if (Button = mbRight) and (rdoMethods.ItemIndex = iiLotsOfLines)
       then // mbRight terminates a polyline
            begin
              // do whatever here
              // clear the marqueerect
              SetLength(LinePoints, 0);
            end; // mbRight polyline/polygon
end;

end.


